# -*- coding: utf-8 -*-
import requests
import re
from typing import List, Dict
from bs4 import BeautifulSoup
from bs4.element import Tag
from urllib.parse import urlencode

BASE_URL = "https://zenodo.org/api/"


class Record:
    def __init__(self, data, zenodo, base_url: str = BASE_URL) -> None:
        self.base_url = base_url
        self.data = data
        self._zenodo = zenodo

    def _row_to_version(self, row: Tag) -> Dict[str, str]:
        link = row.select("a")[0]
        linkrec = row.select("a")[0].attrs["href"]
        if not linkrec:
            raise KeyError("record not found in parsed HTML")

        texts = row.select("small")
        recmatch = re.match(r"/record/(\d*)", linkrec)
        if not recmatch:
            raise LookupError("record match not found in parsed HTML")

        recid = recmatch.group(1)

        return {
            "recid": recid,
            "name": link.text,
            "doi": texts[0].text,
            "date": texts[1].text,
            "original_version": self._zenodo.get_record(recid).original_version(),
        }

    def get_versions(self) -> list:
        url = f"{self.base_url}srecords?all_versions=1&size=100&q=conceptrecid:{self.data['conceptrecid']}"

        print(url)

        data = requests.get(url).json()

        return [Record(hit, self._zenodo) for hit in data["hits"]["hits"]]

    def get_versions_from_webpage(self) -> list:
        """Get version details from Zenodo webpage (it is not available in the REST api)"""
        res = requests.get("https://zenodo.org/record/" + self.data["conceptrecid"])
        soup = BeautifulSoup(res.text, "html.parser")
        version_rows = soup.select(".well.metadata > table.table tr")
        if len(version_rows) == 0:  # when only 1 version
            return [
                {
                    "recid": self.data["id"],
                    "name": "1",
                    "doi": self.data["doi"],
                    "date": self.data["created"],
                    "original_version": self.original_version(),
                }
            ]
        return [self._row_to_version(row) for row in version_rows if len(row.select("td")) > 1]

    def original_version(self):
        for identifier in self.data["metadata"]["related_identifiers"]:
            if identifier["relation"] == "isSupplementTo":
                return re.match(r".*/tree/(.*$)", identifier["identifier"]).group(1)
        return None

    def __str__(self):
        return str(self.data)


class Zenodo:
    def __init__(self, api_key: str = "", base_url: str = BASE_URL) -> None:
        self.base_url = base_url
        self._api_key = api_key
        self.re_github_repo = re.compile(r".*github.com/(.*?/.*?)[/$]")

    def search(self, search: str) -> List[Record]:
        """search Zenodo record for string `search`

        :param search: string to search
        :return: Record[] results
        """
        search = search.replace("/", " ")  # zenodo can't handle '/' in search query
        params = {"q": search}

        recs = self._get_records(params)

        if not recs:
            raise LookupError(f"No records found for search {search}")

        return recs

    def _extract_github_repo(self, identifier):
        matches = self.re_github_repo.match(identifier)

        if matches:
            return matches.group(1)

        raise LookupError(f"No records found with {identifier}")

    def find_record_by_github_repo(self, search: str):
        records = self.search(search)
        for record in records:
            if "metadata" not in record.data or "related_identifiers" not in record.data["metadata"]:
                continue

            for identifier in [identifier["identifier"] for identifier in record.data["metadata"]["related_identifiers"]]:
                repo = self._extract_github_repo(identifier)

                if repo and repo.upper() == search.upper():
                    return record

        raise LookupError(f"No records found in {search}")

    def find_record_by_doi(self, doi: str):
        params = {"q": f"conceptdoi:{doi.replace('/', '*')}"}
        records = self._get_records(params)

        if len(records) > 0:
            return records[0]
        else:
            params = {"q": "doi:%s" % doi.replace("/", "*")}
            return self._get_records(params)[0]

    def get_record(self, recid: str) -> Record:

        url = self.base_url + "records/" + recid

        return Record(requests.get(url).json(), self)

    def _get_records(self, params: Dict[str, str]) -> List[Record]:
        url = self.base_url + "records?" + urlencode(params)

        return [Record(hit, self) for hit in requests.get(url).json()["hits"]["hits"]]



#!/usr/bin/env python
""" examples of searching Zenodo"""
import pyzenodo3


def main():
    zen = pyzenodo3.Zenodo()

    Recs = zen.search("scivision")
    assert Recs is not None
    # %%
    Rec = zen.find_record_by_github_repo("scivision/lowtran")

    vers = Rec.get_versions_from_webpage()

    print(vers)


if __name__ == "__main__":
    main()



""" Zenodo uploads from Python"""
from pathlib import Path
import json
import requests
from configparser import ConfigParser
from typing import Dict, Union, List

from .base import BASE_URL

HDR = {"Content-Type": "application/json"}


def meta(inifn: Path) -> Path:
    """ creates metadata for Zenodo upload.
    1. create dict() with metadata
    2. convert dict() to json
    3. write JSON to disk
    """
    SECT = "zenodo"
    inifn = Path(inifn).expanduser()

    Meta: Dict[str, dict] = {"metadata": {}}
    meta: Dict[str, Union[str, List[str], Dict[str, str]]] = {}

    C = ConfigParser(inline_comment_prefixes=("#", ";"))
    C.read(inifn)

    for k in C[SECT].keys():
        if k == "license":
            meta["license"] = {"id": C.get(SECT, k)}
            continue

        m = C.get(SECT, k)

        if "," in m:
            meta[k] = m.split(",")
        else:
            meta[k] = m

    # %% assemble JSON output
    Meta["metadata"] = meta
    # indent is optional, for human readability
    json_meta = json.dumps(Meta, indent="\t")

    outfn = inifn.with_suffix(".json")

    print("writing JSON metadata to", outfn)
    outfn.write_text(json_meta)

    return outfn


def check_token(token: str, base_url: str):
    if not isinstance(token, str) or not token:
        raise TypeError("Token need to be a string")

    r = requests.get(f"{base_url}/deposit/depositions", params={"access_token": token})

    if r.status_code != 200:
        raise requests.HTTPError(f"Token accept error, status code: {r.status_code}  {r.json()['message']}")


def get_token(token: Union[str, Path]) -> str:
    if Path(token).expanduser().is_file():
        token = Path(token).expanduser().read_text().strip()  # in case \n or spaces sneak in
    elif isinstance(token, str) and 100 > len(token) > 10:
        pass
    else:
        raise ValueError("API Token must be specified to upload to Zenodo")

    return token


def upload_meta(token: str, metafn: Path, depid: str):
    """upload metadata to zenodo"""

    if not metafn:
        raise ValueError("must specify API token or file containing the token")

    metafn = Path(metafn).expanduser()

    if not metafn.is_file():
        raise FileNotFoundError("meta JSON file is required")

    meta = metafn.read_text()

    r = requests.put(
        f"{BASE_URL}/deposit/depositions/{depid}", params={"access_token": token}, data=meta, headers=HDR  # json.dumps(meta),
    )

    if r.status_code != 200:
        raise requests.HTTPError(f"Error in metadata upload, status code: {r.status_code}   {r.json()['message']}")


def upload_data(token: str, datafn: Path, depid: str, base_url: str):

    r = requests.post(
        f"{base_url}/deposit/depositions/{depid}/files",
        params={"access_token": token},
        data={"filename": str(datafn)},
        files={"file": datafn.open("rb")},
    )

    if r.status_code != 201:
        raise requests.HTTPError(f"Error in data upload, status code: {r.status_code}   {r.json()['message']}")

    print(f"{datafn} ID = {depid} (DOI: 10.5281/zenodo.{depid})")


def create(token: str, base_url: str) -> str:

    r = requests.post(f"{base_url}/deposit/depositions", params={"access_token": token}, json={}, headers=HDR)

    if r.status_code != 201:
        raise requests.HTTPError(f"Error in creation, status code: {r.status_code}   {r.json()['message']}")
    # %% Get the deposition ID
    return r.json()["id"]


def upload(metafn: Path, datafn: Path, token: Union[str, Path], base_url=BASE_URL):
    """takes metadata and file and uploads to Zenodo"""

    datafn = Path(datafn).expanduser()
    assert datafn.is_file(), "for now, upload a file only"

    # %% token check
    token = get_token(token)

    check_token(token, base_url)
    # %% Create new submission
    depid = create(token, base_url)
    # %% Upload data
    upload_data(token, datafn, depid, base_url)

    # %% add metadata
    # upload_meta(token, metafn, depid)



#!/usr/bin/env python
"""
inspired by https://github.com/darvasd/upload-to-zenodo/
"""
from argparse import ArgumentParser, Namespace
import pyzenodo3.upload as zup
from pyzenodo3.base import BASE_URL


def cmdparse() -> Namespace:
    p = ArgumentParser(description="Upload data to Zenodo staging")
    p.add_argument("apikey", help="Zenodo API key", nargs="?")
    p.add_argument("inifn", help="mymeta.ini file with author, title, etc.")
    p.add_argument("path", help="directory or file to upload to Zenodo", nargs="?")
    p.add_argument("--use-sandbox", help="Use sandbox.zenodo.org instead of the real site.", action='store_true')
    return p.parse_args()


def main():
    p = cmdparse()

    metafn = zup.meta(p.inifn)

    if p.use_sandbox:
        base_url = "https://sandbox.zenodo.org/api"
    else:
        base_url = BASE_URL

    zup.upload(metafn, p.path, p.apikey, base_url=base_url)


if __name__ == "__main__":
    main()
